# -*- coding: utf-8 -*-
from __future__ import unicode_literals, division, print_function

from django.forms import ModelForm

from multiupload.fields import MultiFileField

from .models import Panorama, PanoramaPhoto


class PanoramaUploadForm(ModelForm):
    """Custom form, providing a MultiFileField widget that allows to add
    multiple photos at the same time.
    """
    photos = MultiFileField(min_num=1, max_num=25, max_file_size=1024*1024*100)

    def save(self, commit=True):
        instance = super(PanoramaUploadForm, self).save(commit)
        for each in self.cleaned_data['photos']:
            PanoramaPhoto.objects.create(photo=each, panorama=instance)
        return instance

    class Meta:
        model = Panorama
        fields = ('name', 'image', 'loop', 'latitude', 'longitude', 'altitude')
